

######################################################################
# Module for posting events to QT GUI-Thread.
# Author: Petteri Aimonen <jpa@kapsi.fi>
# Licenced under GNU General Public Licence
# Part of the KBlackBoard distribution



import threading,qt,traceback

class EventHandler( qt.QObject ):
	def event(self, event):
		if event.type() != qt.QEvent.User:
			return qt.QObject.event(self, event)
		else :
			try :
				event.result.append( event.function( *event.args, **event.kwargs ) )
			except :
				print "Error in GUIEvent :"
				traceback.print_exc()
			else :
				event.resultevent.set()
			
			return True

def sendGUIEvent( function, args, kwargs = {}, timeout = None ):
	'''GUI-Thread will call function with arguments from tuple args and
	keyword-arguments from optional dictionary kwargs. If optional timeout
	is not provided or is None, blocks the running thread until event is
	processed. If processing timeouts, returns None. Otherwise returns
	the return-value from function.'''
	
	handler = EventHandler()
	
	event = qt.QEvent( qt.QEvent.User )
	event.function = function
	event.args = args
	event.kwargs = kwargs
	event.result = [] # Simple object to pass the result in
	event.resultevent = threading.Event()
	
	qt.QThread.postEvent( handler, event )
		
	event.resultevent.wait( timeout )
	
	if len( event.result ) > 0 :
		return event.result[0]
	else:
		return None



if __name__ == "__main__":
	import thread,time
	#Unit testing
	
	def testFunction(app) :
		app.quit() #Happens in the GUI-thread
	
	def testThread(app) :
		sendGUIEvent( testFunction, [app] )
		print "testThread going down"
	
	
	app = qt.QApplication([])
	
	thread.start_new_thread( testThread, (app,) )
	
	app.exec_loop()
	
	print "Main thread got back"
	
	time.sleep(1)
	
	print "Main thread terminating"
	