
from qt import *
import os

from frmFile import frmFile
from configFile import KBlackBoardConfig


class filesPopup( QPopupMenu ):
	
	'''Handles popup for lstFiles and also the related actions'''
	
	def __init__( self , parent , fileslist , name = None ):
		QPopupMenu.__init__( self , parent , name )
		self.parent = parent
		self.fileslist = fileslist
		
		self.insertItem( self.__tr("New") ,  self.new , 0 , 1 )
		self.insertItem( self.__tr("View") , self.view , 0 , 2 )
		self.insertItem( self.__tr("Edit") , self.edit , 0 , 3 )
		self.insertItem( self.__tr("Print") , self.printer , 0 , 4 )
		self.insertItem( self.__tr("Properties") , self.properties , 0 , 5 )
		
		self.insertSeparator( 6 )
		
		self.insertItem( self.__tr("Send to all clients") , 7 )
		self.insertItem( self.__tr("Send to selected clients") , 8 )
		
		self.insertSeparator( 9 )
		
		self.insertItem( self.__tr("Load from file") , self.load , 0 , 10)
		self.insertItem( self.__tr("Save to file") , self.save , 0 , 11 )
		self.insertItem( self.__tr("Duplicate") , self.duplicate, 0 , 12 )
		self.insertItem( self.__tr("Remove") , self.remove, 0 , 13 )
		
		self.insertSeparator( 14 )
		
		self.insertItem( self.__tr("Save list") , self.savelist , 0 , 15 )
		
		self.insertItem( self.__tr("Load list") , self.loadlist , 0 , 16 )
		self.insertItem( self.__tr("Clear list") , self.clearlist , 0 , 17 )
		
		self.itemSpecific = [ 2, 3, 4, 5, 7, 8, 11, 12, 13 ] # Enable these only when item is selected
		
	
	
	
	def popup( self , item , point ) :
		
		if item :
			gotItem = True
		else :
			gotItem = False
		
		for idx in self.itemSpecific :
			self.setItemEnabled( idx , gotItem )

		QPopupMenu.popup( self , point )
	
	def properties( self ):
		f = self.fileslist.getFile( self.parent.currentItem() )
		dialog = frmFile()
		
		dialog.txtName.setText( f.name )
		dialog.txtLog.setText( f.log )
		
		
		if dialog.exec_loop() :
			f.name = str(dialog.txtName.text())
	
	def new( self ):
		
		program, ok = QInputDialog.getItem(
			"KBlackBoard",
			self.__tr("Select application for creating new file:"),
			QStringList.split(';',KBlackBoardConfig.get('Common','Programs')),
			0 , True )
		
		if not ok :
			return
		
		dialog = frmFile()
		if not dialog.exec_loop():
			return
		
		f = self.fileslist.addFile(
			name = str(dialog.txtName.text()),
			program = str(program))
		
		f.edit()
		
	def edit( self ):
		f = self.fileslist.getFile( self.parent.currentItem() )
		f.edit()
	
	def view( self ):
		f = self.fileslist.getFile( self.parent.currentItem() )
		f.view()
	
	def printer( self ):
		f = self.fileslist.getFile( self.parent.currentItem() )
		f.printer()
	
	def load( self ):
		file = str( QFileDialog.getOpenFileName( os.path.expanduser('~'),
			self.__tr("All files (*)"), None ) )
		
		if len( file ) == 0:
			return
		
		dialog = frmFile()
		if not dialog.exec_loop():
			return
		
		f = self.fileslist.loadFile(
			file = file,
			name = str(dialog.txtName.text()))
	
	def save( self ):
		f = self.fileslist.getFile( self.parent.currentItem() )
		
		n = str( QFileDialog.getSaveFileName( os.path.expanduser('~'),
			self.__tr("All files (*)"), None ) )
		
		if len( n ) == 0:
			return
		
		file = open( n, 'wb' )
		
		file.write( f.data )
	
	def savelist( self ):
		file = str( QFileDialog.getSaveFileName( os.path.expanduser('~'),
			self.__tr("KBlackBoard list files (*.kbb)"), None ) )
	
		if len( file ) == 0:
			return
		
		self.fileslist.saveList( file )
	
	def loadlist( self ):
		file = str( QFileDialog.getOpenFileName( os.path.expanduser('~'),
			self.__tr("KBlackBoard list files (*.kbb)"), None ) )
		
		if len( file ) == 0:
			return
		
		self.fileslist.loadList( file )
	
	def remove( self ):
		f = self.fileslist.getFile( self.parent.currentItem() )
		self.fileslist.remove( f )
	
	def duplicate( self ):
		f = self.fileslist.getFile( self.parent.currentItem() )
		d = f.saveState()
		self.fileslist.addFile( dict = d )

	
	def clearlist( self ):
		v = QMessageBox.warning( None,
					"KBlackBoard",
					self.__tr("Warning: All unsaved data will be lost!"),
					QMessageBox.Ok | QMessageBox.Default,
					QMessageBox.Cancel | QMessageBox.Escape )
		
		if v == QMessageBox.Ok :
			self.fileslist.clear()
	
	def __tr(self,s,c = None):
		return qApp.translate("frmMain",s,c)